module Lib where

import Control.Applicative
import Control.Monad
import System.Random


data Gen a = MkGen { unGen :: StdGen -> Int -> a }

instance Functor Gen where
  fmap f (MkGen h) =
    MkGen (\r n -> f (h r n))

instance Applicative Gen where
  pure x =
    MkGen (\_ _ -> x)
  (<*>) = ap

instance Monad Gen where
  return = pure

  -- m :: StdGen -> Int -> a
  -- k :: a -> Gen b
  -- k :: a -> (StdGen -> Int -> b)
  MkGen m >>= k =
    MkGen (\r n ->
      case split r of
        (r1, r2) ->
          let MkGen m' = k (m r1 n)
          in m' r2 n
    )

generate :: Gen a -> IO a
generate (MkGen f) = do
  r <- getStdGen
  return $ f r 42

generateMany :: Int -> Gen a -> IO [a]
generateMany n g = do
  r <- getStdGen
  let MkGen f = replicateM n g
  return $ f r 42

chooseInt :: (Int, Int) -> Gen Int
chooseInt bounds =
  MkGen (\g _ -> fst $ uniformR bounds g)

oneof :: [Gen a] -> Gen a
oneof [] = error "QuickCheck.oneof used with empty list"
oneof gs = chooseInt (0,length gs - 1) >>= (gs !!)
