module Lib2 where

import Control.Monad
import Lib

data Result = Success
            | Failure String 
  deriving (Eq, Show)

class Testable prop where
  property :: prop -> Gen Result

instance Testable Bool where
  property True  = return Success
  property False = return $ Failure ""

quickCheck :: Testable prop => prop -> IO ()
quickCheck p = do
  res <- generateMany 100 $ property p
  putStrLn $ show res

forAll :: (Show a, Testable b) => Gen a -> (a -> b) -> Gen Result
forAll g f = do 
  a <- g
  r <- property (f a)
  return $ case r of
             Success -> Success
             Failure s -> Failure (show a ++ " " ++ s)

class Arbitrary a where
  arbitrary :: Gen a

instance Arbitrary Int where
  arbitrary = chooseInt (-10,10)

instance (Show a, Arbitrary a, Testable b) => Testable (a -> b) where
  property f = forAll arbitrary f


